; corn flakes with bananas.

.sdsctag 0.1, "corn flakes", "with bananas", "chris read"

.memorymap
 defaultslot 0
 slotsize $8000 ; 48KB (ROM)
 slot 0 $0000
 slotsize $2000 ; 8KB (RAM)
 slot 1 $2000
 slotsize $100 ; RAM
 slot 2 $dd00
 slotsize $4000 ; ROM
 slot 3 $8000

.endme

.rombankmap
bankstotal 1
banksize $8000 ; 32KB
banks 1
.endro

; Map of the sprite attribute table (sat) buffer.
; Contains sprites' vertical position (vpos), horizontal posi-
; tion (hpos) and character codes (cc).

.equ gametimervp $c028        ; first score vpos.
.equ gametimerhp $c0d0          ; first score hpos.
.equ gametimercc $c0d1          ; first score cc.

.equ bananavp $c008           ; mr. ultra vpos.
.equ bananahp $c090           ; mr. ultra hpos.
.equ bananacc $c091           ; mr. ultra cc.

.equ endspr $c038          ; first unused sprite.

 ; Organize ram - create the sat buffer and variables.

.enum $c000 export         ; export labels to symbol file.

       satbuf dsb 256      ; sprite attribute table buffer.
                           ; see the map for object offsets.

        iflag db            ; frame interrupt flag.
       input db            ; input from player 1 controller.
       status db           ; vdp status (for collision detect.).


       bananax db              ; Player y.
       bananay db              ; Player x

       controllerispressed db

.ende

.bank 0 slot 0
.org 0
       di                  ; disable interrupts.
       im 1                ; interrupt mode 1.
       ld sp,$dff0         ; default stack pointer address.
       jp inigam           ; initialize game.

; Read the vdp status flag at every frame interrupt.
; Sprite collision is read from bit 5 of the status flag.

.orga $0038                ; frame interrupt address.
        ex af,af'           ; save accumulator in its shadow reg.
        in a,$bf            ; get vdp status / satisfy interrupt.
        ld (status),a       ; save vdp status in ram.
        ld a,1              ; load accumulator with raised flag.
        ld (iflag),a        ; set interrupt flag.
        ex af,af'           ; restore accumulator.
        ei                  ; enable interrupts.
        reti                ; return from interrupt.

; Disable the pause button - this is an unforgiving game!

.orga $0066                ; pause button interrupt.
       retn                ; disable pause button.

; Initialize game.
; Overwrite the first 4 kb of ram with zeroes.

inigam
       ld hl,$c000         ; point to beginning of ram.
       ld bc,$1000         ; 4 kb to fill.
       ld a,0              ; with value 0.
       call mfill          ; do it!

; Use the initialized ram to clean all of vram.

       ld hl,$0000         ; prepare vram for data at $0000.
       call vrampr
       ld b,4              ; write 4 x 4 kb = 16 kb.
-      push bc             ; save the counter.
       ld hl,$c000         ; source = freshly initialized ram.
       ld bc,$1000         ; 4 kb of zeroes.
       call vramwr         ; purge vram.
       pop bc              ; retrieve counter.
       djnz -

 ; Initialize PSGLib.
             call PSGInit

; Initialize the VDP registers.

       ld hl,regdat        ; point to register init data.
       ld b,11             ; 11 bytes of register data.
       ld c,$80            ; VDP register command byte.

-:     ld a,(hl)           ; load one byte of data into A.
       out ($bf),a         ; output data to VDP command port.
       ld a,c              ; load the command byte.
       out ($bf),a         ; output it to the VDP command port.
       inc hl              ; inc. pointer to next byte of data.
       inc c               ; inc. command byte to next register.
       djnz -              ; jump back to '-' if b > 0.

; Put the sprites in.
       ld hl,$c000         ; color bank 2, color 0 (sprites).
       call vrampr         ; prepare vram.
       ld hl,titlescreenpal         ; sprite palette data.
       ld bc,32            ; 5 colors.
       call vramwr         ; set sprite palette.

ldmain

       ld a,%10100000      ; turn screen on - normal sprites.
       ld b,1
       call setreg         ; set register 1.

       ei

       ld hl,$c000         ; color bank 2, color 0 (sprites).
       call vrampr         ; prepare vram.
       ld hl,titlescreenpal         ; sprite palette data.
       ld bc,32            ; 5 colors.
       call vramwr         ; set sprite palette.

       ld hl,$0000         ; first tile @ index 0.
       call vrampr         ; prepare vram.
       ld hl,titlescreentile     ; background tile data (the road).
       ld bc,83*32         ; each tile is 32 bytes.
       call vramwr         ; write background tiles to vram.

       ld hl,$3800         ; point to name table.
       call vrampr         ; prepare vram.
       ld hl,titlescreenmap         ; point to background tilemap data.
       ld bc,32*28*2       ; 32 x 28 tiles, each is 2 bytes.
       call vramwr         ; write name table to vram.


       ld a,%11100000      ; turn screen on - normal sprites.
       ld b,1
       call setreg         ; set register 1.

       ei

       ld hl,tssong        ; load title screen tune.
       call PSGPlay

titlescreenloop  halt
       call PSGFrame

       ;call upbuf

       ;call ldsat          ; load sat buffer to vram. The cars
                           ; and the (hi)-score sprites are
                           ; updated on the screen.

       in     a,($00)
       ;cpl                    ; invert logic so 1 = pressed
       and    %10000000

       jp z,begingame           ; no - test for left key.
       jp titlescreenloop            ; jump back for another round.

begingame
            call PSGStop
       
         ; put bg in
       ld hl,$c000         ; color bank 2, color 0 (sprites).
       call vrampr         ; prepare vram.
       ld hl,levelbgpal         ; sprite palette data.
       ld bc,32            ; 5 colors.
       call vramwr         ; set sprite palette.

       ld hl,$0000         ; first tile @ index 0.
       call vrampr         ; prepare vram.
       ld hl,level1tile     ; background tile data (the road).
       ld bc,83*32         ; each tile is 32 bytes.
       call vramwr         ; write background tiles to vram.

       ld hl,$3800         ; point to name table.
       call vrampr         ; prepare vram.
       ld hl,level1map         ; point to background tilemap data.
       ld bc,32*28*2       ; 32 x 28 tiles, each is 2 bytes.
       call vramwr         ; write name table to vram.

       ;put bananasprite in

       ld hl,$2000         ; first tile @ index 272.
       call vrampr         ; prepare vram.
       ld hl,mrbantile    ; cloud tile data.
       ld bc,4*32          ; 16 tiles, 32 bytes each.
       call vramwr         ; write enemy car tiles to vram.

       ld de,bananacc         ; point to player cc in buffer.
       ld hl,mrbanmap        ; point to player car graphics.
       call bananaccthing          ; set the char codes for player car.

       ld hl,$c020         ; color bank 2, color 0 (sprites).
       call vrampr         ; prepare vram.
       ld hl,spritepal        ; sprite palette data.
       ld bc,32            ; 5 colors.
       call vramwr         ; set sprite palette.

         ld a,0         ;
       ld (controllerispressed),a    

       ld a,52          ;
       ld (bananax),a          ; set player y.

       ld a,23         ;
       ld (bananay),a          ; set player y.

maingameloop
        call wait               ; start main loop with vblank.

       call ldsat          ; load sat buffer to vram. The cars
                           ; and the (hi)-score sprites are
                           ; updated on the screen.

       ;call PSGFrame

       call upbuf

move_left
       call getkey         ; read controller port.
       ld a,(input)        ; read input from ram mirror.
       bit 2,a             ; is left key pressed?
       jp nz,move_right       ; no - end key check.

       ld a,(controllerispressed)          ; get player's hpos (x-coordinate).
       cp 1           ; is player over the left border?
       jp z,no_key_test       ; yes - then don't move left.

       ld a,(bananax)          ; get player's hpos (x-coordinate).
       cp 53           ; is player over the left border?
       jp c,no_key_test         ; yes - then don't move left.

       ld a,1         ;
       ld (controllerispressed),a

       ld a,(bananax)          ; get player x-coordinate.
       ld b, 16        ; load horizontal speed (constant).
       sub b               ; subtract hspeed from player x.
       ld (bananax),a          ; update player x-coordinate.

       jp no_key_test

move_right
       call getkey         ; read controller port.
       ld a,(input)        ; read input from ram mirror.
       bit 3,a             ; is right key pressed?
       jp nz,move_up       ; no - end key check.

       ld a,(controllerispressed)          ; get player's hpos (x-coordinate).
       cp 1           ; is player over the left border?
       jp z,no_key_test       ; yes - then don't move left.

       ld a,(bananax)          ; get player's hpos (x-coordinate).
       cp 196         ; is player over thr right border?
       jp nc,no_key_test         ; yes - skip to left test.

       ld a,1         ;
       ld (controllerispressed),a

       ld a,(bananax)          ; get player x-coordinate.
       add a,16        ; add constant hspeed
       ld (bananax),a          ; update player x-coordinate.

              jp no_key_test

move_up
       call getkey         ; read controller port.
       ld a,(input)        ; read input from ram mirror.
       bit 0,a             ; is left key pressed?
       jp nz,move_down      ; no - end key check.

         ld a,(controllerispressed)          ; get player's hpos (x-coordinate).
       cp 1           ; is player over the left border?
       jp z,no_key_test       ; yes - then don't move left.

       ld a,(bananay)          ; get player's hpos (x-coordinate).
       cp 24           ; is player over the left border?
       jp c,no_key_test       ; yes - then don't move left.

       ld a,1         ;
       ld (controllerispressed),a

       ld a,(bananay)          ; get player x-coordinate.
       ld b, 16        ; load horizontal speed (constant).
       sub b               ; subtract hspeed from player x.
       ld (bananay),a          ; update player x-coordinate.
              jp no_key_test

move_down
       call getkey         ; read controller port.
       ld a,(input)        ; read input from ram mirror.
       bit 1,a             ; is right key pressed?
       jp nz,no_key_test   ; no - end key check.

       ld a,(controllerispressed)          ; get player's hpos (x-coordinate).
       cp 1           ; is player over the left border?
       jp z,no_key_test       ; yes - then don't move left.

       ld a,(bananay)          ; get player's hpos (x-coordinate).
       cp 135         ; is player over thr right border?
       jp nc,maingameloop         ; yes - skip to left test.

       ld a,1         ;
       ld (controllerispressed),a

       ld a,(bananay)          ; get player x-coordinate.
       add a,16        ; add constant hspeed
       ld (bananay),a          ; update player x-coordinate.

no_key_test
       call getkey         ; read controller port.
       ld a,(input)        ; read input from ram mirror.
       bit 1,a             ; is right key pressed?
       jp z,maingameloop    ; no - end key check.

       call getkey         ; read controller port.
       ld a,(input)        ; read input from ram mirror.
       bit 0,a             ; is right key pressed?
       jp z,maingameloop    ; no - end key check.

       call getkey         ; read controller port.
       ld a,(input)        ; read input from ram mirror.
       bit 3,a             ; is right key pressed?
       jp z,maingameloop    ; no - end key check.

       call getkey         ; read controller port.
       ld a,(input)        ; read input from ram mirror.
       bit 2,a             ; is right key pressed?
       jp z,maingameloop    ; no - end key check.

       ld a,0         ;
       ld (controllerispressed),a

          jp maingameloop

; --------------------------------------------------------------
; SET VDP REGISTER.
; Write to target register.
; A = byte to be loaded into vdp register.
; B = target register 0-10.

setreg out ($bf),a         ; output command word 1/2.
       ld a,$80
       or b
       out ($bf),a         ; output command word 2/2.
       ret	
	   
; --------------------------------------------------------------
; LOAD SPRITE ATTRIBUTE TABLE
; Load data into sprite attribute table (SAT) from the buffer.

ldsat  ld hl,$3f00         ; point to start of SAT in vram.
       call vrampr         ; prepare vram to recieve data.
       ld b,255            ; amount of bytes to output.
       ld c,$be            ; destination is vdp data port.
       ld hl,satbuf        ; source is start of sat buffer.
       otir                ; output buffer to vdp.
       ret

; --------------------------------------------------------------
; MEMORY FILL.
; HL = base address, BC = area size, A = fill byte.

mfill  ld (hl),a           ; load filler byte to base address.
       ld d,h              ; make DE = HL.
       ld e,l
       inc de              ; increment DE to HL + 1.
       dec bc              ; decrement counter.
       ld a,b              ; was BC = 0001 to begin with?
       or c
       ret z               ; yes - then just return.
       ldir                ; else - write filler byte BC times,
                           ; while incrementing DE and HL...
       ret

; PREPARE VRAM.
; Set up vdp to recieve data at vram address in HL.

vrampr push af
       ld a,l
       out ($bf),a
       ld a,h
       or $40
       out ($bf),a
       pop af
       ret

;----
upbuf
       ld a,(bananay)          ; load player's current y-coordinate.
       ld hl,bananavp         ; point to sat buffer.
       call bananaything           ; refresh buffer according to y.

       ld a,(bananax)          ; load player's current x-coordinate.
       ld hl,bananahp         ; point to sat buffer.
       call bananaxthing           ; refresh buffer according to x.

       ret

; --------------------------------------------------------------
; CAR Y TO SPRITES' VERTICAL POSITIONS (VPOS) IN BUFFER.
; Generate vpos sat buffer data from a car's y position.
; A = car's y (i.e. ply), HL = buffer address of car vpos.

bananaything   ld c,a              ; save hpos in C
       .rept 2             ; wladx: Repeat code four times.
       ld a,c              ; load hpos into A
       ld b,2             ; loop: Repeat four times.
-      ld (hl),a           ; write value to buffer at address.
       inc hl              ; skip over the char code byte.
       inc hl              ; point to next hpos byte in buffer.
       add a,8             ; add 8 (a tile's width in pixels).
       djnz -              ; jump back
       .endr               ; end of wladx repeat directive.
       ret
; --------------------------------------------------------------
; CAR X TO SPRITES' HORIZONTAL POSITIONS (HPOS) IN BUFFER.
; Generates hpos sat buffer data from a car's x position.
; A = car's x (i.e. plx), HL = buffer address of car hpos.

bananaxthing ld c,a              ; save hpos in C
       .rept 2             ; wladx: Repeat code four times.
       ld a,c              ; load hpos into A

       ld b,2             ; loop: Repeat four times.
-      ld (hl),a           ; write value to buffer at address.
       inc hl              ; skip over the char code byte.
       inc hl              ; point to next hpos byte in buffer.
       add a,8             ; add 8 (a tile's width in pixels).
       djnz -              ; jump back
       .endr               ; end of wladx repeat directive.
       ret

; --------------------------------------------------------------
; SET CAR SPRITES' CHARACTER CODES (CC)
; HL = pointer to 16 byte char codes block, DE = buffer index.

bananaccthing ld bc,32
-      ldi
       inc de
       ld a,b
       or c
       jp nz,-
       ret

; --------------------------------------------------------------
; GET KEYS.
; Read player 1 keys (port $dc) into ram mirror (input).

getkey in a,$dc            ; read player 1 input port $dc.
       ld (input),a        ; let variable mirror port status.
       ret

; --------------------------------------------------------------
; WRITE TO VRAM
; Write BC amount of bytes from data source pointed to by HL.
; Tip: Use vrampr before calling.

vramwr ld a,(hl)
       out ($be),a
       inc hl
       dec bc
       ld a,c
       or b
       jp nz,vramwr
       ret

;-----------
upbuf3

	ret

; Wait for vertical blanking phase.
wait   ld a,(iflag)        ; load frame interrupt flag.
       or a                ; is it set?
       jp z,wait           ; no? - keep looping.
       xor a               ; else - reset flag.
       ld (iflag),a
       ret                 ; return.

; Initial values for the 11 vdp registers.

regdat .db %00000110       ; reg. 0, display and interrupt mode.
                           ; bit 4 = line interrupt (disabled).
                           ; 5 = blank left column (disabled).
                           ; 6 = hori. scroll inhibit (disabled).
                           ; 7 = vert. scroll inhibit (disabled).

       .db %10100000       ; reg. 1, display and interrupt mode.
                           ; bit 0 = zoomed sprites (enabled).
                           ; 1 = 8 x 16 sprites (disabled).
                           ; 5 = frame interrupt (enabled).
                           ; 6 = display (blanked).

       .db $ff             ; reg. 2, name table address.
                           ; $ff = name table at $3800.

       .db $ff             ; reg. 3, n.a.
                           ; always set it to $ff.

       .db $ff             ; reg. 4, n.a.
                           ; always set it to $ff.

       .db $ff             ; reg. 5, sprite attribute table.
                           ; $ff = sprite attrib. table at $3F00.

       .db $ff             ; reg. 6, sprite tile address.
                           ; $ff = sprite tiles in bank 2.

       .db %11110011       ; reg. 7, border color.
                           ; set to color 3 in bank 2.

       .db $00             ; reg. 8, horizontal scroll value = 0.

       .db $00             ; reg. 9, vertical scroll value = 0.

       .db $ff             ; reg. 10, raster line interrupt.
                           ; turn off line int. requests.

; Initialization for hiscore and score in the sat buffer.


mrbanmap .dw $0800
         .dw $0801


titlescreenpal .include "stufftoadd/biggergg (palette).inc"
titlescreentile .include "stufftoadd/biggergg (tiles).inc"
titlescreenmap .include "stufftoadd/biggergg (tilemap).inc"

spritepal .include "stufftoadd/mrban (palette).inc"
mrbantile .include "stufftoadd/mrban (tiles).inc"

levelbgpal .include "stufftoadd/level1bg (palette).inc"
level1tile .include "stufftoadd/level1 (tiles).inc"
level1map .include "stufftoadd/level1 (tilemap).inc"


tssong .incbin "stufftoadd\mrbananaats3.psg"
.include "stufftoadd\PSGlib.inc"